
; Routinen fuer CPD6 RAM-Floppy


; ******************************************************************

; theoretische Betrachtungen zur RAM-Floppy
;
; die RAM-Floppy beginnt ab 10000H und geht bis 7FFFFH. Das sind 70000H (458752) Byte Platz.
;
; bei 128 Byte je Sektor sind das = 458752 / 128 = 3584 Sektoren

; Auf jeder Spur befinden sich 72 Sektoren -> 3584 / 72 = 49 Spuren (abgerundet)

; 49 * 72 * 128 = 451584 (6E400H) Byte. Es sind 1C00H (7168) Byte (56 Sektoren) ungenutzt (keine vollstaendige Spur).

; Wichtig: Zu allen Adresswerten kommt fuer die RAM-Floppy ein Offsett von 10000H,
;          da sich im Adressbereic 00000H bis 0FFFFH (64K) das CP/M-System befindet

; ******************************************************************


; *** ERA_RFL ***
;
; Aufruf: ERA_RFL
;
; Funktion:
; Die gesammte RAM-Floppy wird mit E5H berschrieben (gelscht). Dazu wird die Write-Funktion genutzt um
; einen vorher mit E5H beschriebenen Sektor auf alle RAM-Floppy Sektoren zu schreiben.
;
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
;
ERA_RFL:

		; 1. erzeugen eines Sektors (an der DMA-Adresse 80H - DMAAD) mit E5H
		;
		LD	A,0E5H
		LD	B,128 
		LD	HL,080H

ERA_RFL_A:
		LD	(HL),A
		INC	HL
		DJNZ	ERA_RFL_A


		; 2. schreiben des Sektors auf alle Spuren und Sektoren der RAM-Floppy

		; Laden DMA-Adresse (bleibt konstant) 
		LD	HL,080H
		LD	(DMAAD),HL

		; Lade Laufwerk B: (bleibt konstant)
		LD	A,0
		LD	(DISKNO),A


		; Spur (0..48)
		LD	BC,0

		LD	DE,(spt1)	; 72 Sektoren pro Spur in E


ERA_RFL_B:

		; Sektor (0..71)
		LD	D,0

		LD	(TRACK),BC


ERA_RFL_C:
		LD	A,D
		LD	(SECTOR),A



		PUSH	BC
		PUSH	DE

		; Loeschen durch Schreiben mit 5EH
		CALL	Write

		POP	DE
		POP	BC


		INC	D
		LD	A,D
		CP	E
		
		JR	NZ,ERA_RFL_C
		;



; -------------------------------------------

		PUSH	BC
		PUSH	DE

		CALL	FORMAT_To_CONOUT

		POP	DE
		POP	BC



; -------------------------------------------




		; Setzen Laufwerk, Spur, Sektor und DMA-Adresse 

		LD	A,0
		LD	(DISKNO),A

		LD	HL,49
		LD	(TRACK),HL

		LD	A,0
		LD	(SECTOR),A

		LD	HL,80H
		LD	(DMAAD),HL


		; Loeschen durch Schreiben mit 5EH
		CALL	Write






		CALL	AUS_CRLF	; Abschluss der Eingabezeile


		; Ruecksprung
		;
		RET





; *** XLOCATE ***
;
; Aufruf: XLOCATE
;
;
; Funktion: Berechnen der RAM-Adresse und der Speicherbank aus TRACK und SECTOR
; 
; Uebergabeparameter: keine
; Rueckgabeparameter: HL = RAM-Adresse an die der Sektor kopiert werden soll
;                     C  = Auswahl der Speicherbank 
;
;
; 76543210 76543210 76543210
; --------------------------
;
;      111 11111111 11111111 = maximal moegliche Adresse
;
;      BBB AAAAAAAA AAAAAAAA
;      876 54321098 76543210
; --------------------------
;              1101 11001000 = maximale moegliche TRACK*SPT = DC8
;              SSSS SSSSSSSS = (TRACK*SPT)+SECTOR = Marke1
;              BBB           = Bank -> Marke3
;
;      SSS SSSSSSSS SRRRRRRR = ((TRACK*SPT)+SECTOR) * RECORD (128 Byte) = Marke2
;      BBB                   = BANK 
;
XLOCATE:

		LD	BC,(TRACK)	; laden der Spurnummer
		LD	B,C
		LD	DE,(spt1)	; SPT1 = 72
		LD	HL,0
		;
		XOR	A
		CP	B
		JR	Z,XLOCATE2	; TRACK*SPT
		;
XLOCATE1:	ADD	HL,DE
		DJNZ	XLOCATE1
		;
XLOCATE2:	LD	A,(SECTOR)
		LD	E,A
		ADD	HL,DE		; (TRACK*SPT)+SECTOR -> Marke1
		;
		LD	C,H
		SRL	C
		INC	C		; BANK (+1 Offset) nach C -> Marke3
		;
		LD	B,7		; Multiplikation mit 128 (7 * Linksschieben)
		;
XLOCATE3:	SLA	L
		RL	H
		DJNZ	XLOCATE3	; -> Marke2
		;
		; Ruecksprung
		;
		RET
















